""" Functions about nori is placed at here. """
import pickle

import cv2
import nori2 as nori
import numpy as np


def get_img(nori_id, img_key='img', is_color=True):
    """ Obtain an image in a given nori_id by key.

    Args:
        nori_id (str): nori id.
        img_key (str): key for img. default: 'img'.
        is_color (bool): if the wanted image is RGB image, default: True.

    Returns:
        np.ndarray: The image in the nori.

    Example:
        >>> nori_id = '995224433,qqreqragagaag'
        >>> img = image.get_img(nori_id, img_key="img", is_color=True)

    """
    fetcher = nori.Fetcher()
    raw = pickle.loads(fetcher.get(nori_id))
    flag = cv2.IMREAD_COLOR if is_color else cv2.IMREAD_GRAYSCALE
    img_raw = cv2.imdecode(np.fromstring(raw[img_key], np.uint8), flag)
    assert img_raw is not None, \
        'img_raw is None, please check the nori_id exists: ' + nori_id
    return img_raw


def get_mask(nori_id, h, w, img_key='mask', is_color=False):
    """ Obtain an image in a given nori_id by key.

    Args:
        nori_id (str): nori id.
        h (int): height of mask.
        w (int): width of mask.
        img_key (str): key for img. default: 'mask'.
        is_color (bool): if the wanted image is RGB image, default: False.

    Returns:
        np.ndarray: The mask in the nori.

    Example:
        >>> nori_id = '995224433,qqreqragagaag'
        >>> img = image.get_mask(nori_id, h=256, w=256, img_key="mask", is_color=True)

    """
    fetcher = nori.Fetcher()
    raw = pickle.loads(fetcher.get(nori_id))
    flag = cv2.IMREAD_COLOR if is_color else cv2.IMREAD_GRAYSCALE
    try:
        img_raw = np.frombuffer(raw[img_key], np.uint8).reshape(h, w)
    except ValueError:
        img_raw = cv2.imdecode(np.frombuffer(raw[img_key], np.uint8), flag)
    assert img_raw is not None, \
        'img_raw is None, please check the nori_id exists: ' + nori_id
    return img_raw



""" Save useful utilities for dozy at here. """
import os


def check_file_exists(file_path, create=False):
    """ Check if a file exists. if `create` is True, create it when didn't exist
    Args:
        file_path (str): file path
        create (bool, optional): if file doesn't exist, whether to create it
    Returns:
        Return True if the file exists, otherwise False

    Example:
        >>> file_path = '/path/to/file.ext'
        >>> utils.check_file_exists(file_path, create=True)
        True

    """
    if os.path.exists(file_path):
        return True
    if create:
        os.mknod(file_path)
    return False


def check_dir_exists(dir_path, create=False):
    """ Check if a dir exists. If `create` is True, create it when didn't exist
    Args:
        dir_path (str): path of a directory
        create (bool, optional), if the dir doesn't exist, whether to create it
    Returns:
        Return True if the dir exists, otherwise False

    Example:
        >>> dir_path = '/path/to/dir'
        >>> utils.check_dir_exists(dir_path, create=True)
        True

    """
    if os.path.exists(dir_path):
        return True
    if create:
        os.makedirs(dir_path)
    return False


def complaint_absence(path):
    """ Throw out an error when a file or a directory doesn't exist
    Args:
        path (str): path of a file or a directory

    Example:
        >>> dir_path = '/path/to/dir'
        >>> utils.complaint_absence(dir_path)
        FileNotFoundError: path/to/dir doesn't exist

    """
    if not os.path.exists(path):
        raise FileNotFoundError('{} doesn\'t exist.'.format(path))


def complaint_file_absence(path):
    """ Throw out an error if the path doesn't exist or is not a file
    Args:
        path (str): path to verify whether is a file

    Example:
        >>> file_path = '/path/to/file'
        >>> utils.complaint_file_absence(file_path)
        Exception: path/to/file is not a file, please check again

    """
    complaint_absence(path)
    if not os.path.isfile(path):
        raise Exception('{} is not a file, please check again'.format(path))


def complaint_dir_absence(path):
    """ Throw out an error if the path doesn't exist or is not a dir
    Args:
        path (str): path to verify whether is a dir

    Example:
        >>> dir_path = '/path/to/dir'
        >>> utils.complaint_dir_absence(dir_path)
        Exception: path/to/dir is not a dir, please check again

    """
    complaint_absence(path)
    if not os.path.isdir(path):
        raise Exception('{} is not a dir, please check again.'.format(path))



""" video module of dozy. """
import glob
import os

import cv2

from . import image
from . import utils


def extract(video_path, frm_save_dir, prefix='', pattern='%06d.jpg'):
    """ Extract frames of a video and save them.

    Args:
        video_path (str): path of a video.
        frm_save_dir (str): where to save the frames.
        prefix (str): prefix of name for saving frames, default: ''.
        pattern (str): format of frame names, default: '%06d.jpg'

    Example:
        >>> video_path = '/path/to/video'
        >>> frm_save_dir = '/path/to/frame/save/dir'
        >>> video.extract(video_path, frm_save_dir)
        All frames have been saved.

    """
    utils.complaint_file_absence(video_path)
    utils.complaint_dir_absence(os.path.dirname(frm_save_dir))

    if not utils.check_dir_exists(frm_save_dir, create=True):
        print('dir {} didn\'t exist, created here for you'.format(frm_save_dir))

    cap = cv2.VideoCapture(video_path)
    idx = 0
    while True:
        _, img = cap.read(0)
        if img is None:
            print('All frames have been saved.')
            break

        img_save_name = prefix + pattern % idx
        img_save_path = os.path.join(frm_save_dir, img_save_name)
        image.save(img_save_path, img)
        idx += 1

    cap.release()


def merge(frm_dir, video_save_path, step=1, fps=25, filter_pattern='*.*', h=None, w=None):
    """ Combine frames to a video file.

    Args:
        frm_dir (str): directory where to find images to make the video.
        video_save_path (str): where to save the created video.
        step (int): Frame interval to generate the video
        fps (int): fps (Frames Per Second), default: 25.
        filter_pattern (str): format to find the targe image in frm_dir,
            default: '*.*'.
        h (int): height of created video. if None, use default height of first
            frame.
        w (int): width of created video. if None, use default width of first
            frame.

    Example:
        >>> frm_dir = '/path/to/frame/dir'
        >>> vdie_save_path = '/path/to/video/save/video.mp4'
        >>> video.merge(frm_dir, video_save_path)
        All images have been written to the video.

    """
    utils.complaint_dir_absence(frm_dir)
    utils.complaint_dir_absence(os.path.dirname(video_save_path))

    img_pattern = os.path.join(frm_dir, filter_pattern)
    img_paths = sorted(glob.glob(img_pattern), key=lambda x: int(x.split('/')[-1].split('.')[0]))[0::step]

    h_init, w_init = cv2.imread(img_paths[0]).shape[:2]
    h = h_init if h is None else h
    w = w_init if w is None else w

    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    writer = cv2.VideoWriter(video_save_path, fourcc, fps, (w, h))
    print('Using fps {} and size {}x{} to save video'.format(fps, h, w))
    for img_path in img_paths:
        frame = image.load(img_path)
        writer.write(frame)
    print('All images have been written to the video.')

    writer.release()


def play(video_path):
    """ Play a video.

    Args:
        video_path (str): the path of the video to play.

    Example:
        >>> video_path = '/path/to/video.mp4'
        >>> video.play(video_path)

    """
    utils.complaint_file_absence(video_path)

    cap = cv2.VideoCapture(video_path)
    while cap.isOpened():
        ret, frm = cap.read()
        if ret:
            cv2.imshow('frame', frm)
            k = cv2.waitKey(1)
            if k & 0xFF == ord('q'):
                break
        else:
            break

    cap.release()


def record(video_save_path, fps=25, h=480, w=640):
    """ record a realtime video using a camera.

    Args:
        video_save_path (str): where to save the video.
        fps (int): how many frames per second, default: 25.
        h (int): height of saved video, default: 480.
        w (int): width of saved video, default: 640.

    Example:
        >>> video.record('/path/to/save/video.mp4')

    """
    utils.complaint_dir_absence(os.path.dirname(video_save_path))

    cap = cv2.VideoCapture(0)
    video_name_extension = os.path.splitext(video_save_path)[-1]
    fourcc_pattern = "XVID" if video_name_extension == '.avi' else "MP4V"
    fourcc = cv2.VideoWriter_fourcc(*fourcc_pattern)
    writer = cv2.VideoWriter(video_save_path, fourcc, fps, (w, h))

    while cap.isOpened():
        ret, frm = cap.read()
        if ret:
            cv2.imshow('frame', frm)
            k = cv2.waitKey(1)
            if k & 0xFF == ord('q'):
                break
            else:
                writer.write(frm)
        else:
            break

    cap.release()
    writer.release()
    cv2.destroyAllWindows()



""" Test module for Dozy. """
import os
import sys
import unittest
from unittest import TestCase

import cv2
import numpy as np

test_dir = os.path.dirname(__file__)
src_dir = os.path.join(test_dir, '..', 'dozy')
sys.path.insert(0, src_dir)

import image
import eval


class TestImage(TestCase):
	def test_image_save(self):
		img = cv2.imread('imgs/cat.jpg')
		save_path1 = './cat1.jpg'
		save_path2 = './none/cat2.jpg'
		self.assertTrue(image.save(save_path1, img))
		self.assertFalse(image.save(save_path2, img))

	def test_image_load(self):
		img_path1 = 'imgs/cat.jpg'
		img_path2 = 'imgs/dog.jpg'
		img1 = cv2.imread(img_path1)
		img2 = cv2.imread(img_path2)
		self.assertEqual(img1, image.load(img_path1))
		self.assertFalse(img1, image.load(img_path2))

	def test_image_show(self):
		pass
	
	def test_image_crop(self):
		img = cv2.imread('imgs/cat.jpg')
		x0, y0, x1, y1 = 10, 10, 20, 20
		self.assertEqual(img, image.crop(img))
		self.assertEqual(img[y0:, :], image.crop(img, y0=y0))
		self.assertEqual(img[:y1, :], image.crop(img, y1=y1))
		self.assertEqual(img[:, x0:], image.crop(img, x0=x0))
		self.assertEqual(img[:, :x1], image.crop(img, x1=x1))

	def test_image_combine(self):
		pass

	def test_image_resize(self):
		pass


if __name__ == '__main__':
	unittest.main()



""" Functions of evaluation for images and videos."""
import glob
import os

import numpy as np

from . import utils
from . import image


def _seg_hist(pred, gt, num_class=2):
    """ create hist.

    Args:
        pred (np.ndarray): Predicted image, has value in range[0, 255] (no
            binarization yet).
        gt (np.ndarray): Ground truth image, has value in range[0, 255] (no
            binarization yet).
        num_class (int, optional): How many classes in each image. Default: 2.

    Returns:
        np.ndarray: The bind of predicted image and ground truth image.

    """
    pred = (pred > 100).astype('uint8')
    gt = (gt > 100).astype('uint8')

    k = (gt >= 0) & (gt < num_class)
    return np.bincount(num_class * gt[k].astype(int) + pred[k].astype(int),
                       minlength=num_class ** 2).reshape(num_class, num_class)


def _seg_score(hist):
    """ calculate mean IoU.

    Args:
        hist (np.ndarray): Get from method _seg_hist.

    Returns:
        iu (np.ndarray): IoU array.
        acc (np.ndarray): Contains value for each pred and gt pair
            mean_iou (np.ndarray): Mean IoU on all pred and gt pairs.

    """
    acc = np.diag(hist).sum() / hist.sum()
    iou = np.diag(hist) / (hist.sum(1) + hist.sum(0) - np.diag(hist))
    mean_iou = np.nanmean(iou)
    return iou, acc, mean_iou


def iu(pred, gt, num_class=2):
    """ calculate Mean IoU for groundtruth image and predicted image.

    Args:
        pred (np.ndarray): Predicted result image, shape [h, w].
        gt (np.ndarray): Groud truth image, shape [h, w].
        num_class (int, optional): How many classes in each image. Default: 2.

    Returns:
        float: IoU value of pred image and gt image.

    Example:
        >>> pred = image.load('/path/to/pred/00001.mask.png')
        >>> gt = image.load('/path/to/gt/00000.png')
        >>> print('iou:', eval.iu(pred, gt))

    """
    hist = _seg_hist(pred, gt, num_class=num_class)
    _, _, iou = _seg_score(hist)
    return iou


def miu(pred_dir, gt_dir, pred_ptn="*.mask.png", gt_ptn="*.png", num_class=2):
    """ calculate Mean IoU for images in groundtruth dir and predicted dir.

    Args:
        pred_dir (str): Where to find predicted images.
        gt_dir (str): Where to find ground truth images.
        pred_ptn (str, optional): The pattern for finding predicted image in a
            dir. Default: `*.mask.png`.
        gt_ptn (str, optional): The pattern for finding gt image in a dir.
            Default: `*.png`.
        num_class (int, optional): How many classes in each image. Default: 2.

    Returns:
        float: Mean IoU for all predicted and ground truth image pairs.

    Example:
        >>> pred_dir = '/path/to/pred/dir'
        >>> gt_dir = '/path/to/gt/dir'
        >>> print('mean-iou:', eval.miu(pred_dir, gt_dir))

    """
    utils.complaint_dir_absence(pred_dir)
    utils.complaint_dir_absence(gt_dir)

    pred_filter_ptn = os.path.join(pred_dir, pred_ptn)
    gt_filter_ptn = os.path.join(gt_dir, gt_ptn)

    pred_img_paths = sorted(glob.glob(pred_filter_ptn))
    gt_img_paths = sorted(glob.glob(gt_filter_ptn))

    assert len(pred_img_paths) == len(gt_img_paths), \
        'nums of pred images and gt images must be equal'

    hist = np.zeros((num_class, num_class))
    for (pred_img_path, gt_img_path) in zip(pred_img_paths, gt_img_paths):
        cur_pred = image.load(pred_img_path)
        cur_gt = image.load(gt_img_path)
        hist += _seg_hist(cur_pred, cur_gt, num_class=num_class)

    _, _, mean_iou = _seg_score(hist)
    return mean_iou



""" Setup scripts for dozy. """
from setuptools import setup, find_packages


setup(
    name="dozy",
    version="0.0.4",
    description="A simple and user-friendly tool for operating images and videos",
    url="https://github.com/vra/dozy",
    author="Yunfeng Wang, Peisen Wang",
    author_email="wyf.brz@gmail.com",
    packages=find_packages()
)



""" image module of dozy. """
import glob
import os

import cv2
import numpy as np

from . import utils


def combine(img1, img2, axis=0):
    r""" Combine two images img1 and img2 to one image horizionly or vertically.

    Args:
        img1 (np.ndarray): first image to combine.
        img2 (np.ndarray): second image to combine.
        axis (int, optional): which axis to combine images. 0 for vertically, 1
            for horizon. default: 0.

    Returns:
        np.ndarray: combined image.

    Example:
        >>> img1 = image.load('/path/to/img1.jpg')
        >>> img2 = image.load('/path/to/img2.jpg')
        >>> img3 = image.combine(img1, img2, axis=0)
        >>> image.show(img3, 'combined image')

    """
    res = np.concatenate((img1, img2), axis=axis)
    return res


def combine_list(imgs, axis=0):
    r""" Combine multiple images to one image horizionly or vertically.

    Args:
        imgs (list): list of np.ndarray, i.e., images to combine.
        axis (int, optional): which axis to combine images. 0 for vertically, 1
            for horizon. default: 0.

    Returns:
        np.ndarray: combined image.

    Example:
        >>> img1 = image.load('/path/to/img1.jpg')
        >>> img2 = image.load('/path/to/img2.jpg')
        >>> img3 = image.load('/path/to/img3.jpg')
        >>> img4 = image.combine([img1, img2, img3], axis=0)
        >>> image.show(img4, 'combined image')

    """
    res = np.concatenate(imgs, axis=axis)
    return res


def crop(img, x0=None, y0=None, x1=None, y1=None):
    """Crop img[y0:y1, x0:x1] from img and return this part.

    Args:
        img (numpy.ndarray): an RGB or grayscale image. For RGB image,  shape
            is [h, w, c]. for grayscale image, shape is [h, w].
        y0 (int): top y coordinates of target part of img to crop.
        y1 (int): bottom y coordinates of target part of img to crop.
        x0 (int): left x coordinates of target part of img to crop.
        x1 (int): right x coordinates of target part of img to crop.

    Returns:
        img[y0:y1, x0:x1].

    Raises:
        ValueError: if value of coordinates < 0.

    Example:
        >>> img = imagel.load('/path/to/img1.jpg')
        >>> sub_img = image.crop(img, x0=10, y=10, x1=200, y1=200)

    """

    h, w = img.shape[:2]
    y0 = 0 if y0 is None else y0
    y1 = h if y1 is None else y1
    x0 = 0 if x0 is None else x0
    x1 = w if x1 is None else x1

    assert isinstance(img, np.ndarray), 'img is not a np.ndarray'
    assert img.ndim in [2, 3], 'img must be RGB image or grayscale image'
    assert isinstance(y0, int), 'y0 is not a int'
    assert isinstance(y1, int), 'y1 is not a int'
    assert isinstance(x0, int), 'x0 is not a int'
    assert isinstance(x1, int), 'x1 is not a int'

    if y0 < 0:
        raise ValueError('y0 is < 0')
    if y1 < 0:
        raise ValueError('y1 is < 0')
    if x0 < 0:
        raise ValueError('x0 is < 0')
    if x1 < 0:
        raise ValueError('x1 is < 0')

    return img[y0:y1, x0:x1]


def load(img_path):
    """ Load an image from a given path.

    Args:
        img_path (str): where to load the image . must have extension.

    Return:
        An image has type np.ndarray.

    Example:
        >>> img = image.load('/path/to/img.jpg')
        >>> print(type(img))
        <class 'numpy.ndarray'>
        >>> print(img.shape)
        (340, 325, 3)

    """
    assert isinstance(img_path, str), 'img_path must be a str'
    utils.complaint_file_absence(img_path)

    img = cv2.imread(img_path)
    return img


def merge_masks(masks):
    """ Merge multiple mask images into one image.

    We assume the mask of images have no intersection. Otherwise, the mask of
    latter image will override the previous images.

    Args:
        masks (list): a list contain multiple grayscale images only containng
        valule 0 and 255, and all have same size

    Returns:
        np.ndarray: One grayscale image

    Example:
        >>> img1 = image.load('/path/to/mask1.png')
        >>> img2 = image.load('/path/to/mask2.png')
        >>> merged_mask = image.merge_masks([img1, img2])

    Raises:
        ValueError: If length of masks is zero.
    """

    assert isinstance(masks, list), 'masks must be a list.'
    if len(masks) < 1:
        raise ValueError('length of masks is zero.')

    res_img = masks[0]
    for img in masks[1:]:
        res_img[img > 0] = img[img > 0]

    return res_img


def merge_masks_folder(folders, ptn="*.*"):
    """ Merge grayscale images in mutiple directories according names.

    Note:
        In order to support any length of folders, we assume the corresponding
        images in different dir have same name (extension can be different).

    Args:
        folders (list): a str list containing abs folder paths.
        ptn (str): the pattern to search wanted images in each folder.

    Raises:
        ValueError: If length of folders is zero.

    Yields:
        np.ndarray: next merged image.
        str: The basename of next operated image.

    Example:
        >>> fd1 = '/path/to/dir1'
        >>> fd2 = '/path/to/dir2'
        >>> for img in merge_masks_folder([fd1, fd2]):
                print(img.shape)

    """
    assert isinstance(folders, list), 'folders must be a list.'
    if len(folders) < 1:
        raise ValueError('length of folder is zero.')
    for fd in folders:
        utils.complaint_dir_absence(fd)

    first_fd = folders[0]
    sample_img_paths = sorted(glob.glob(os.path.join(first_fd, ptn)))
    for img_path in sample_img_paths:
        first_ext = os.path.splitext(img_path)[1]
        print(img_path, first_ext)
        img = cv2.imread(img_path, cv2.IMREAD_GRAYSCALE)
        imgs = [img]
        for cur_fd in folders[1:]:
            cur_ext = os.path.splitext(os.listdir(cur_fd)[0])[1]
            print(cur_ext)
            cur_img_path = img_path.replace(first_fd, cur_fd).replace(
                first_ext, cur_ext)
            print(cur_img_path)
            cur_img = cv2.imread(cur_img_path, cv2.IMREAD_GRAYSCALE)
            imgs.append(cur_img)

        yield merge_masks(imgs), os.path.basename(img_path)


def resize(img, h, w):
    """ Resize img to target shape [h, w].

    Args:
        img (np.ndarray): image to resize.
        h (int): height of target image.
        w (int): width of target image.

    Returns:
        np.ndarray with shape [h, w] or [h, w, 3].

    Example:
        >>> img = image.load('/path/to/img.jpg')
        >>> img_re = image.resize(img, h=256, w=256)

    """

    assert isinstance(h, int), 'h must be an int'
    assert isinstance(w, int), 'w must be an int'
    assert h > 0, 'h must be larger than 0'
    assert w > 0, 'h must be larger than 0'

    return cv2.resize(img, (w, h))


def save(save_path, img):
    """ Save an image.

    Args:
        save_path (str): where to save the image. must have extension.
        img (np.ndarray): an image has shape [h, w, c] or [h, w].

    Returns:
        return True if save successfully else False.

    Example:
        >>> img = image.load(/path/to/img.jpg')
        >>> image.save('/path/to/img_new.jpg', img)

    """
    assert isinstance(save_path, str), 'save_path must be a str'
    utils.complaint_dir_absence(os.path.dirname(save_path))

    res = cv2.imwrite(save_path, img)

    if res is None:
        print('Image saving failed. Please check whethe the save_path exists.')
        return False
    return True


def show(img, show_name='img', delay=0):
    """ Show an image.

    Args:
        img (np.ndarray): an image has shape [h, w, c] or [h, w].
        show_name (str, optional): the name of the show window. Default: 'img'.
        delay (int, optional): how many milliseconds to wait the window then
            close. 0 means forever util key pressed. Default: 0.

    Example:
        >>> img = image.load('/path/to/img.jpg')
        >>> image.show(img, show_name="img")

    """
    assert isinstance(img, np.ndarray), 'img must be a np.ndarray'

    cv2.imshow(show_name, img)
    cv2.waitKey(delay)
    cv2.destroyAllWindows()


def show_pair(img1, img2, show_name="img_pair", delay=0):
    """ Show two images in a row.

    Args:
        img1 (np.ndarray): First image to show.
        img2 (np.ndarray): Second image to show.
        show_name (str, optional): name for the window to show image. Default:
            'img_pair'.
        delay (int, optional): how many milliseconds to wait the window then
            close. 0 means forever util key pressed. default: 0

    Example:
        >>> img1 = image.load('/path/to/img1.jpg')
        >>> img2 = image.load('/path/to/img2.jpg')
        >>> image.show_pair(img1, img2, show_name="img_pair")

    """
    img_combine = combine(img1, img2, axis=1)
    show(img_combine, show_name=show_name, delay=delay)


def show_pair_folder(dir1, dir2, dir1_ptn="*.*", dir2_ptn="*.*"):
    r""" Show images in two folder to compare them easily.

    Args:
        dir1 (str): path of first directory.
        dir2 (str): path of second directory.
        dir1_ptn(str, optional): pattern for filtering image in dir1. Default:
            '*.*'
        dir2_ptn(str, optional): pattern for filtering image in dir2. Default:
            '*.*'

    Example:
        >>> dir1 = image.load('/path/to/dir1')
        >>> dir2 = image.load('/path/to/dir2')
        >>> image.show_pair_folder(img1, img2, show_name="img_pair_folder")

    """
    utils.complaint_dir_absence(dir1)
    utils.complaint_dir_absence(dir2)

    dir1_full_pth = os.path.join(dir1, dir1_ptn)
    dir2_full_pth = os.path.join(dir2, dir2_ptn)
    imgs_path1 = sorted(glob.glob(dir1_full_pth))
    imgs_path2 = sorted(glob.glob(dir2_full_pth))
    assert len(imgs_path1) == len(imgs_path2), 'images in two dir must be equal'

    def update(imgs_path1, imgs_path2, idx):
        """ Update shown images. """
        img1 = load(imgs_path1[idx])
        img2 = load(imgs_path2[idx])
        img_comb = combine(img1, img2, axis=1)
        cv2.imshow('img_comb', img_comb)

    idx = 0
    update(imgs_path1, imgs_path2, idx)
    while True:
        ch = cv2.waitKeyEx()
        if ch & 0xFF == ord('q'):
            break
        if ch == 65361 or ch == 65362 or ch == ord('h') or ch == ord('k'):
            idx = max(0, idx-1)
            update(imgs_path1, imgs_path2, idx)
            idx -= 1
        elif ch == 65363 or ch == 65364 or ch == ord('l') or ch == ord('j'):
            idx = min(idx, len(imgs_path1))
            update(imgs_path1, imgs_path2, idx)
            idx += 1
        else:
            print(ch)

    cv2.destroyAllWindows()
